# Generated by Django 2.0.7 on 2018-07-26 17:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('phenotype_ontologies', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ontology',
            name='type',
            field=models.PositiveSmallIntegerField(choices=[(1, 'HP'), (2, 'MONDO'), (3, 'ONCOTREE')]),
        ),
    ]



from django.conf import settings


HPO_PURL = getattr(
    settings,
    'HPO_PURL',
    'http://purl.obolibrary.org/obo/hp.obo'
)


MONDO_PURL = getattr(
    settings,
    'MONDO_PURL',
    'http://purl.obolibrary.org/obo/mondo.obo'
)


NCIT_PURL = getattr(
    settings,
    'NCIT_PURL',
    'http://purl.obolibrary.org/obo/ncit/ncit-oncotree.obo'
)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _

from model_utils import Choices


ONTOLOGY = Choices(
    (1, 'HP', _('HP')),  # http://purl.obolibrary.org/obo/hp.obo
    (2, 'MONDO', _('MONDO')),  # http://purl.obolibrary.org/obo/mondo.obo
    (3, 'ONCOTREE', _('ONCOTREE')),  # http://purl.obolibrary.org/obo/ncit/ncit-oncotree.owl
)


SYNONYM_SCOPES = Choices(
    (1, 'EXACT', _('EXACT')),
    (2, 'BROAD', _('BROAD')),
    (3, 'NARROW', _('NARROW')),
    (4, 'RELATED', _('RELATED')),
    (5, 'ABBREVATION', _('ABBREVATION')),
)



# -*- coding: utf-8 -*-
from django.db.models import QuerySet


class TermQuerySet(QuerySet):

    def fast(self):
        return self.select_related('ontology') \
            .prefetch_related('xrefs') \
            .prefetch_related('synonyms') \
            .prefetch_related('relationships__type') \
            .all()


class SynonymQuerySet(QuerySet):

    def fast(self):
        return self.select_related('term').all()


class CrossReferenceQuerySet(QuerySet):

    def fast(self):
        return self.select_related('term').all()


class RelationshipQuerySet(QuerySet):

    def fast(self):
        return self.select_related('type').select_related('term').all()



# -*- coding: utf-8 -*-
from django.db.models import CharField, TextField

import django_filters
from genomix.filters import DisplayChoiceFilter

from . import choices, models


class TermFilter(django_filters.rest_framework.FilterSet):

    ontology__type = DisplayChoiceFilter(choices=choices.ONTOLOGY)

    ontology = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Ontology.objects.all(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    synonyms = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Synonym.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    xrefs = django_filters.ModelMultipleChoiceFilter(
        queryset=models.CrossReference.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    relationships = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Relationship.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    relationships__related_term = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    class Meta:
        model = models.Term
        fields = [
            'ontology',
            'ontology__type',
            'ontology__label',
            'identifier',
            'label',
            'description',
            'synonyms',
            'synonyms__description',
            'xrefs',
            'xrefs__source',
            'xrefs__source_value',
            'relationships',
            'relationships__type__label',
            'relationships__related_term',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }


class CrossReferenceFilter(django_filters.rest_framework.FilterSet):

    term = django_filters.ModelMultipleChoiceFilter(
        queryset=models.Term.objects.fast(),
        widget=django_filters.widgets.CSVWidget,
        help_text='Multiple values may be separated by commas.',
    )

    class Meta:
        model = models.CrossReference
        fields = [
            'term',
            'source',
            'source_value',
        ]
        filter_overrides = {
            CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            }
        }
